"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const path_1 = require("path");
const command = (config, db, dao, service, ...args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('Downgrading database...');
    yield db.transaction((client) => __awaiter(void 0, void 0, void 0, function* () {
        yield dao.initialize(client);
        // Lock all concurrent writes, but allow concurrent reads
        yield dao.acquireExclusiveLock(client);
        // Revert current revision
        const { finalRevision, pendingRevisionModules } = yield service.downgrade(client, {
            namespace: config.revisionsNamespace,
            directory: config.revisionsDirectory
        });
        const initialRevisionModule = pendingRevisionModules[0];
        if (pendingRevisionModules.length === 0) {
            console.log('...nothing to downgrade');
        }
        else if (finalRevision === undefined) {
            console.log(`version: ${initialRevisionModule.version} -> (base)`);
            console.log(`file:    ${(0, path_1.basename)(initialRevisionModule.file)} -> (base)`);
        }
        else {
            console.log('version: ' +
                initialRevisionModule.version +
                ' -> ' +
                finalRevision.version);
            console.log('file:    ' +
                (0, path_1.basename)(initialRevisionModule.file) +
                ' -> ' +
                (0, path_1.basename)(finalRevision.file));
        }
        // Unlock resource
        yield dao.releaseExclusiveLock(client);
    }));
});
exports.command = command;
