"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const path_1 = require("path");
const command = (config, db, dao, service, ...args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('Upgrading database...');
    yield db.transaction((client) => __awaiter(void 0, void 0, void 0, function* () {
        yield dao.initialize(client);
        // Lock all concurrent writes, but allow concurrent reads
        yield dao.acquireExclusiveLock(client);
        // Apply all pending revisions
        const { initialRevision, pendingRevisionModules } = yield service.upgrade(client, {
            namespace: config.revisionsNamespace,
            directory: config.revisionsDirectory
        });
        const finalRevisionModule = pendingRevisionModules[pendingRevisionModules.length - 1];
        if (pendingRevisionModules.length === 0) {
            console.log('...nothing to upgrade');
        }
        else if (initialRevision === undefined) {
            console.log(`version: (base) -> ${finalRevisionModule.version}`);
            console.log(`file:    (base) -> ${(0, path_1.basename)(finalRevisionModule.file)}`);
        }
        else {
            console.log('version: ' +
                initialRevision.version +
                ' -> ' +
                finalRevisionModule.version);
            console.log('file:    ' +
                (0, path_1.basename)(initialRevision.file) +
                ' -> ' +
                (0, path_1.basename)(finalRevisionModule.file));
        }
        // Unlock resource
        yield dao.releaseExclusiveLock(client);
    }));
});
exports.command = command;
