"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoDBConnectionManager = void 0;
const mongoose_1 = __importDefault(require("mongoose"));
const logger_1 = require("../lib/logger");
class MongoDBConnectionManager {
    constructor(connection, options) {
        var _a;
        this.connection = connection;
        this.logger = (_a = options === null || options === void 0 ? void 0 : options.logger) !== null && _a !== void 0 ? _a : (0, logger_1.getLogger)(MongoDBConnectionManager.name);
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.warn('MongoDB client is shutting down');
            yield mongoose_1.default.disconnect();
        });
    }
    ping() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug('MongoDB client is pinging server');
            if (this.connection.readyState !== mongoose_1.default.STATES.connected) {
                throw new Error('mongodb client not connected');
            }
        });
    }
    transaction(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            // Reference: https://mongoosejs.com/docs/transactions.html
            const session = yield this.connection.startSession();
            try {
                this.logger.debug('Beginning transaction');
                session.startTransaction();
                this.logger.debug('Running transaction callback');
                // eslint-disable-next-line n/no-callback-literal
                const result = yield callback({
                    connection: this.connection,
                    session
                });
                this.logger.debug('Committing transaction');
                yield session.commitTransaction();
                this.logger.debug('Committed transaction');
                return result;
            }
            catch (error) {
                this.logger.warn('Rolling back transaction');
                yield session.abortTransaction();
                throw error;
            }
            finally {
                this.logger.debug('Ending transaction');
                yield session.endSession();
            }
        });
    }
    static createConnection(uri) {
        return mongoose_1.default.createConnection(uri);
    }
}
exports.MongoDBConnectionManager = MongoDBConnectionManager;
